/** @file Filter.h

  Class definitions for Filter-based Devices.

  Your device will typically be derived from one of the base types in this header

  Choosing which parent to derive from is mostly about providing intelligent defaults and allowing the application
  to sort the devices appropriately. Internally World Machine is almost completely agnostic as to the actual class of the device.

  */

#pragma once
#if !defined(AFX_FILTER_H__7ECDF35C_FE0C_47B4_9845_02BF3998EEA9__INCLUDED_)
#define AFX_FILTER_H__7ECDF35C_FE0C_47B4_9845_02BF3998EEA9__INCLUDED_

#include "device.h"
#include "PluginHeader.h"

/// Filters modify an input (possibly consuming additional inputs) and produce an output
///	@ingroup devices
class  Filter : public Device  
{
public:
	Filter();
	virtual dword Color() { return 0x0d5935e; }; 
	virtual int GetDeviceFamily() { return TYPE_FILTER; };
	virtual int getSpeedClass() { return 4; };
	virtual bool isMaskable() { return true; };		
	/// Default filter behavior is to mask its primary output against the primary input, and then to mask the extra outputs against zero. Override this implementation if you wish to change that!
	virtual bool doMaskCreateNewCopies(std::vector<PacketPointer > &results,  BuildContext &context);
};

/// Combine multiple inputs together in some fashion
///	@ingroup devices
class  CombineFilter : public Filter {
public:
	CombineFilter();
	virtual dword Color() { return 0x0b0723d; }; 
	virtual int GetDeviceFamily() { return TYPE_COMBINER; };
	virtual bool isMaskable() { return false; };
	virtual int getSpeedClass() { return 4; };
};

/// Filters that attempt to reproduce effects from the natural world.
///	@ingroup devices
class  NaturalFilter : public Filter {
public:
	NaturalFilter();
	virtual dword Color() { return 0x0456f9c; }; 
	virtual int GetDeviceFamily() { return TYPE_NATURAL; };
	virtual int getSpeedClass() { return 0; };
};

/// Create a mask from the input based upon some set of criteria
///	@ingroup devices
class Selector : public Device  
{
public:
	Selector();
	virtual dword Color() { return 0x0965096; }; 
	virtual int GetDeviceFamily() { return TYPE_SELECTOR; };

	virtual int getSpeedClass() { return 5; };
	virtual bool isMaskable() { return true; };
protected:
	// Default selector behavior is to mask all outputs against zero.
	virtual bool doMaskCreateNewCopies(std::vector<PacketPointer > &results,  BuildContext &context);
};

/// 'plumbing' or otherwise miscellaneous devices
///	@ingroup devices
class Utility : public Device {
public:
	Utility();
	virtual dword Color() { return 0x06699ff; }; 
	virtual int GetDeviceFamily() { return TYPE_UTILITY; };

	virtual int getSpeedClass() { return 5; };
	virtual bool isMaskable() { return false; };

	/// Force implementation of proxy function -- this is becuase the utility devices generally speaking handle packets differently --
	/// they route rather than process packets and so the output sizes may not be the same as the inputs (and the inputs may be different from each other)
	virtual void doCreateOutputProxy( BuildContext &proxycontext )  = 0;

};

/// Change from one type of data to another (Such as greyscale->RGB)
///	@ingroup devices
class Converter : public Device {
public:
	Converter();
	virtual dword Color() { return 0x04f6f4f; }; 
	virtual int GetDeviceFamily() { return TYPE_CONVERTER; };

	virtual bool isMaskable() { return false; };
};


#endif // !defined(AFX_FILTER_H__7ECDF35C_FE0C_47B4_9845_02BF3998EEA9__INCLUDED_)
